//++=======================================================================
// Program Name:  NetWare Redirector for Windows NT
// File Name:     NWCAPI.H
// Version:       v1.00
// Authors:       Brady Anderson    (banderso) (940307)
//
// Abstract:      This module contains all typedefs and constants for the
//                NetWare Client APIs.
//
// Notes:         This interface is accessed using the Win32
//                DeviceIoControl API.
//
// Revision History:
//
//
// (C) Unpublished Copyright of Novell, Inc.  All Rights Reserved.
//
// No part of this file may be duplicated, revised, translated, localized,
// or modified in any manner or compiled, linked or uploaded or downloaded
// to or from any computer system without the prior written consent of
// Novell, Inc.
//=======================================================================--

//=======================================================================

#ifndef _NWCAPI_
#define _NWCAPI_

//
// Network Name Format Type
//

#define  NWC_NAME_FORMAT_NDS              0x0001
#define  NWC_NAME_FORMAT_BIND             0x0002
#define  NWC_NAME_FORMAT_BDP              0x0004
#define  NWC_NAME_FORMAT_NDS_TREE         0x0008
#define  NWC_NAME_FORMAT_WILD             0x8000

//
// API String Types
//

#define  NWC_STRING_TYPE_ASCII            0x0001
#define  NWC_STRING_TYPE_UNICODE          0x0002

/* Transport Type - (nuint32 value) */
#define NWC_TRAN_TYPE_IPX                 0x0001
#define NWC_TRAN_TYPE_DDP                 0x0003
#define NWC_TRAN_TYPE_ASP                 0x0004
#define NWC_TRAN_TYPE_UDP                 0x0008
#define NWC_TRAN_TYPE_TCP                 0x0009
#define NWC_TRAN_TYPE_UDP6                0x000A
#define NWC_TRAN_TYPE_TCP6                0x000B
#define NWC_TRAN_TYPE_WILD                0x8000

//
// Open Connection Flags
//

#define  NWC_OPEN_LICENSED                0x0001
#define  NWC_OPEN_UNLICENSED              0x0002
#define  NWC_OPEN_PRIVATE                 0x0004
#define  NWC_OPEN_PUBLIC                  0x0008
#define  NWC_OPEN_EXISTING_HANDLE         0x0010
#define  NWC_OPEN_NO_HANDLE               0x0020
#define  NWC_OPEN_PERMANENT               0x0040
#define  NWC_OPEN_DISCONNECTED            0x0080
#define  NWC_OPEN_NEAREST                 0x0100
#define  NWC_OPEN_IGNORE_CACHE            0x0200

//
// Close Connection Flags
//

#define  NWC_CLOSE_TEMPORARY              0x0000
#define  NWC_CLOSE_PERMANENT              0x0001

//
// Connection Information Levels
//

#define  NWC_CONN_INFO_RETURN_ALL            0xFFFF
#define  NWC_CONN_INFO_RETURN_NONE           0x0000
#define  NWC_CONN_INFO_VERSION               0x0001
#define  NWC_CONN_INFO_AUTH_STATE            0x0002
#define  NWC_CONN_INFO_BCAST_STATE           0x0003
#define  NWC_CONN_INFO_CONN_REF              0x0004
#define  NWC_CONN_INFO_TREE_NAME             0x0005
#define  NWC_CONN_INFO_WORKGROUP_ID          0x0006
#define  NWC_CONN_INFO_SECURITY_STATE        0x0007
#define  NWC_CONN_INFO_CONN_NUMBER           0x0008
#define  NWC_CONN_INFO_USER_ID               0x0009
#define  NWC_CONN_INFO_SERVER_NAME           0x000A
#define  NWC_CONN_INFO_TRAN_ADDR             0x000B
#define  NWC_CONN_INFO_NDS_STATE             0x000C
#define  NWC_CONN_INFO_MAX_PACKET_SIZE       0x000D
#define  NWC_CONN_INFO_LICENSE_STATE         0x000E
#define  NWC_CONN_INFO_PUBLIC_STATE          0x000F
#define  NWC_CONN_INFO_SERVICE_TYPE          0x0010
#define  NWC_CONN_INFO_DISTANCE              0x0011
#define  NWC_CONN_INFO_SERVER_VERSION        0x0012
#define  NWC_CONN_INFO_AUTH_ID               0x0013
#define  NWC_CONN_INFO_SUSPENDED             0x0014
#define  NWC_CONN_INFO_TREE_NAME_UNICODE     0x0015
#define  NWC_CONN_INFO_SERVER_NAME_UNICODE   0x0016
#define  NWC_CONN_INFO_LOCAL_TRAN_ADDR       0x0017
#define  NWC_CONN_INFO_ALTERNATE_ADDR        0x0018
#define  NWC_CONN_INFO_SERVER_GUID           0x0019

#define  NWC_CONN_INFO_MAX_LEVEL             0x0014

//
// Information Versions
//

#define  NWC_INFO_VERSION_1               0x0001
#define  NWC_INFO_VERSION_2               0x0002

//
// Authentication State
//

#define  NWC_AUTH_TYPE_NONE               0x0000
#define  NWC_AUTH_TYPE_BINDERY            0x0001
#define  NWC_AUTH_TYPE_NDS                0x0002
#define  NWC_AUTH_TYPE_PNW                0x0003

#define  NWC_AUTH_STATE_NONE              0x0000
#define  NWC_AUTH_STATE_BINDERY           0x0001
#define  NWC_AUTH_STATE_NDS               0x0002
#define  NWC_AUTH_STATE_PNW               0x0003

//
// Authentication Flags
//

#define  NWC_AUTH_PRIVATE                 0x00000004
#define  NWC_AUTH_PUBLIC                  0x00000008

//
// Broadcast State
//

#define  NWC_BCAST_PERMIT_ALL             0x0000
#define  NWC_BCAST_PERMIT_SYSTEM          0x0001
#define  NWC_BCAST_PERMIT_NONE            0x0002
#define NWC_BCAST_PERMIT_SYSTEM_POLLED  0x0003
#define NWC_BCAST_PERMIT_ALL_POLLED             0x0004

//
// Security State
//

#define  NWC_SECURITY_SIGNING_NOT_IN_USE  0x00000000
#define  NWC_SECURITY_SIGNING_IN_USE      0x00000001
#define  NWC_SECURITY_LEVEL_CHECKSUM      0x00000100
#define  NWC_SECURITY_LEVEL_SIGN_HEADERS  0x00000200
#define  NWC_SECURITY_LEVEL_SIGN_ALL      0x00000400
#define  NWC_SECURITY_LEVEL_ENCRYPT       0x00000800

//
// Broadcast State
//

#define  NWC_NDS_NOT_CAPABLE              0x0000
#define  NWC_NDS_CAPABLE                  0x0001

//
// License State
//

#define  NWC_NOT_LICENSED                 0x0000
#define  NWC_CONNECTION_LICENSED          0x0001
#define  NWC_HANDLE_LICENSED              0x0002

//
// Public State
//

#define  NWC_CONN_PUBLIC                  0x0000
#define  NWC_CONN_PRIVATE                 0x0001

//
// Scan Connection Information Flags used
// for finding connections by specific criteria
//

#define  NWC_MATCH_NOT_EQUALS             0x0000
#define  NWC_MATCH_EQUALS                 0x0001
#define  NWC_RETURN_PUBLIC                0x0002
#define  NWC_RETURN_PRIVATE               0x0004
#define  NWC_RETURN_LICENSED              0x0008
#define  NWC_RETURN_UNLICENSED            0x0010

//
// Authentication Types
//

#define  NWC_AUTHENT_BIND                 0x0001
#define  NWC_AUTHENT_NDS                  0x0002
#define  NWC_AUTHENT_PNW                  0x0003

//
// Disconnected info
//

#define  NWC_SUSPENDED                    0x0001

//
// Print Flags
//

#define  PRINT_FLAG_RELEASE               0x00000001
#define  PRINT_FLAG_SUPPRESS_FORM_FEED    0x00000002
#define  PRINT_FLAG_TEXT_FILE             0x00000004
#define  PRINT_FLAG_PRINT_BANNER          0x00000008
#define  PRINT_FLAG_NOTIFY_JOB_COMPLETE   0x00000010

//
// Lengths for Print APIs
//

#define  MAX_JOBDESCR_LENGTH              50
#define  MAX_FORM_NAME_LENGTH             13
#define  MAX_BANNER_LENGTH                13
#define  MAX_QUEUE_NAME_LENGTH            65

//
// Maximum object lengths
//

#define  MAX_DEVICE_LENGTH                16
#define  MAX_NETWORK_NAME_LENGTH          1024
#define  MAX_OBJECT_NAME_LENGTH           48
#define  MAX_PASSWORD_LENGTH              128
#define  MAX_SERVER_NAME_LENGTH           48
#define  MAX_SERVICE_TYPE_LENGTH          48
#define  MAX_TREE_NAME_LENGTH             32
#define  MAX_ADDRESS_LENGTH               32
#define  MAX_NAME_SERVICE_PROVIDERS       10

//
// TDS flags used when allocating a tagged data storage node
//

#define  TDS_PRE_ZERO                     0x00000001
#define  TDS_POST_ZERO                    0x00000002
#define  TDS_ENCRYPT                      0x00000004

#define  TDS_VALID_FLAGS                  0x00000007

//
// Flags for the GetBroadcastMessage API
//

#define  MESSAGE_GET_NEXT_MESSAGE         1
#define  MESSAGE_RECEIVED_FOR_CONNECTION  2

//
// Global Events (Types)
//

#define  EVENT_CONN_CREATE                0x00000001
#define  EVENT_CONN_TERMINATE             0x00000002
#define  EVENT_CONN_STATE_CHANGE          0x00000004
#define  EVENT_DEVICE_MAP                 0x00000008
#define  EVENT_DEVICE_UNMAP               0x00000010
#define  EVENT_BROADCAST_NOTIFY           0x00000100
#define  EVENT_NDS_AUTHENTICATE           0x00001000

#define  MAX_GLOBAL_EVENT_DATA            512
#define  MAX_BROADCAST_MESSAGE_LENGTH     255

//
// Device Types
//

#define  DEVICE_DISK                      0x00000001
#define  DEVICE_PARALLEL_PORT             0x00000002
#define  DEVICE_SERIAL_PORT               0x00000003

//
// This constant must always be equal to the last device
//

#define  DEVICE_LAST_DEVICE               0x00000003

//
// Defined feature set provided by requester
//

#ifndef  NWC_FEAT_PRIV_CONN
#define  NWC_FEAT_PRIV_CONN               1
#define  NWC_FEAT_REQ_AUTH                2
#define  NWC_FEAT_SECURITY                3
#define  NWC_FEAT_NDS                     4
#define  NWC_FEAT_NDS_MTREE               5
#define  NWC_FEAT_PRN_CAPTURE             6
#define  NWC_FEAT_NDS_RESOLVE             7
#endif

//
// Name resolution type definitions for NWC_RESOLVE_OBJECT_COMPONENTS
//

#define  RESOLVE_TO_DS                    0x00000001
#define  RESOLVE_TO_FILE_SYSTEM           0x00000002

//
// Link status definitions
//

#define  LINK_STATUS_NETWORK_AVAILABLE    0x00000001

//===[ Type definitions ]==================================================

//
// Global event packet
//

typedef  struct   tagGlobalEvent
{
   //
   // The event type(s) the caller is waiting for...
   //

   unsigned long          EventRegistration;

   //
   // The event type received
   //

   unsigned long          EventReceived;

   //
   // Data returned that is specific to the event type
   //

   union
   {
      struct
      {
         unsigned long       ConnectionReference;
         unsigned long       MessageLength;
         unsigned char        Message[MAX_BROADCAST_MESSAGE_LENGTH];
         unsigned long       ServerLength;
         unsigned char        Server[MAX_SERVER_NAME_LENGTH+1];

      } Broadcast;

      struct
      {
         //
         // Maximum size of the union
         //

         unsigned char        ReservePacket[MAX_GLOBAL_EVENT_DATA];

      } Reserved;

   } GlobalEvent;

} GlobalEvent, *PGlobalEvent;

//
// Connection Handle returned from all OpenConnByXXXX calls
//

typedef  unsigned long    NW_CONN_HANDLE, *PNW_CONN_HANDLE;

//
// Authentication Id returned from the NwcCreateAuthenticationId call
//

typedef  unsigned long    AUTHEN_ID, *PAUTHEN_ID;

#ifndef _NCPLTYPE_H_

#ifndef WCHAR
typedef  unsigned short int WCHAR, *PWCHAR;
#endif

#ifndef PCHAR
typedef char CHAR, *PCHAR;
#endif

#ifndef VOID
typedef void VOID, *PVOID;
#endif
#endif

#ifndef LONG
typedef long LONG, *PLONG;
#endif

//
// Capture Read/Write structure definition
//

//
// Structure for defining what a transport
// address looks like
//

typedef  struct   tagNwcTranAddr
{
   unsigned long       uTransportType;
   unsigned long       uAddressLength;
   unsigned char       *puAddress;

} NwcTranAddr, *PNwcTranAddr;

//
// Structure for defining what a new transport
// address looks like
//

typedef  struct   tagNwcTranAddrEx
{
   unsigned long                   uTransportType;
   unsigned long                   uAddressLength;
   unsigned char                   buBuffer[MAX_ADDRESS_LENGTH];

} NwcTranAddrEx, *PNwcTranAddrEx;

typedef  struct   tagNwcReferral
{
   unsigned long                   uAddrCnt;
   PNwcTranAddrEx          pAddrs;

} NwcReferral, *PNwcReferral;

typedef  struct   tagNwcServerVersion
{
   unsigned long                   uMajorVersion;
   unsigned short int                  uMinorVersion;
   unsigned short int                  uRevision;

} NwcServerVersion, *PNwcServerVersion;

typedef  struct   tagNwcConnString
{
   PCHAR             pString;
   unsigned long     uStringType;
   unsigned long     uNameFormatType;

} NwcConnString, *PNwcConnString;

typedef  struct   tagNwcString
{
   unsigned long    DataType;
   unsigned long    BuffSize;
   unsigned long    DataLen;
   void				*pBuffer;
   unsigned long    CodePage;
   unsigned long    CountryCode;

} NwcString, *PNwcString;

//
// Structure used in NDS Resolve name
//

#define  RESOLVE_INFO_SVC_V1_00     0x00FE0001

typedef  struct   tagNwcResolveInfo
{
   unsigned long      uResolveInfoVersion;
   unsigned long      luFlags;
   unsigned long      luReqFlags;
   unsigned long      luReqScope;
   unsigned long      luResolveType;
   unsigned long      luRepFlags;
   unsigned long      luResolvedOffset;
   unsigned long      luDerefNameLen;
   PWCHAR             pDerefName;

} NwcResolveInfo, *PNwcResolveInfo;

//
// Definition of a fragment for the Raw NCP requests
//

typedef  struct   tagNwcFrag
{
   void				*pData;
   unsigned long    uLength;

} NwcFrag, *PNwcFrag;

//
// Current connection information available for
// enumeration using GetConnInfo and ScanConnInfo
//

typedef  struct   tagNwcConnInfo
{
   unsigned long                   uInfoVersion;
   unsigned long                   uAuthenticationState;
   unsigned long                   uBroadcastState;
   unsigned long                   uConnectionReference;
   PCHAR                   pTreeName;
   PCHAR                   pWorkGroupId;
   unsigned long                   uSecurityState;
   unsigned long                   uConnectionNumber;
   unsigned long                   uUserId;
   PCHAR                   pServerName;
   PNwcTranAddr            pTranAddress;
   unsigned long                   uNdsState;
   unsigned long                   uMaxPacketSize;
   unsigned long                   uLicenseState;
   unsigned long                   uPublicState;
   PCHAR                   pServiceType;
   unsigned long                   uDistance;
   PNwcServerVersion       pServerVersion;
   unsigned long                   uAuthId;
   unsigned long                   uDisconnected;

} NwcConnInfo, *PNwcConnInfo;


//
// The following are Authentication Information
// structure for the different types of authenticators
//

// BINDERY
typedef  struct   tagAuthenBinderyInfo
{
   unsigned long                   ObjectType;

} AuthenBinderyInfo, *PAuthenBinderyInfo;

// NDS
typedef  struct   tagAuthenNdsInfo
{
   PCHAR                   pTreeName[MAX_TREE_NAME_LENGTH];

} AuthenNdsInfo, *PAuthenNdsInfo;

// PNW
typedef  struct   tagAuthenPnwInfo
{
   unsigned long                   ObjectType;

} AuthenPnwInfo, *PAuthenPnwInfo;

//
// Connection table statistics
//

typedef  struct   tagConnTableStatistics
{
   //
   // Total number of successful connects over the
   // current up-time existence of the redirector.
   //

   unsigned long          TotalSuccessfulConnects;

   //
   // Total number of failed connects over the
   // current up-time existence of the redirector
   //

   unsigned long          TotalFailedConnects;

   //
   // The maximum number of connections in the CONN_ATTACHED
   // state over the current up-time existence of the redirector.
   //

   unsigned long          MaxConcurrentConnections;

   //
   // The current number of connections in the attached state
   //

   unsigned long          CurrentConnections;

   //
   // The current number of slots available for connections
   // before the index array is extended.
   //

   unsigned long          CurrentAllocatedSlots;

   //
   // The total number of connection handles allocated
   // in the system.
   //

   unsigned long          TotalAllocatedHandles;

   //
   // The current number of connection handles
   // sitting in the connection handle free list.
   //

   unsigned long          TotalHandlesOnFreeList;

} ConnTableStatistics, *PConnTableStatistics;

//
// Connection statistics
//

typedef  struct   tagConnStatistics
{
   //
   // Total number of successful reconnects
   //

   unsigned long          TotalSuccessfulReconnects;

   //
   // Total number of failed reconnects
   //

   unsigned long          TotalFailedReconnects;

   //
   // Total number of transport failures on
   // this connection.
   //

   unsigned long          TotalTransportFailures;

   //
   // Total number of retries on this
   // connection.
   //

   unsigned long          TotalRetries;

   //
   // Total number of times the retry
   // count was exhausted on this
   // connection.
   //

   unsigned long          TotalRetryExhaustions;

   //
   // Total number of server busy packets
   // received on this connection.
   //

   unsigned long          ServerBusyPackets;

   //
   // Current number of open files on
   // this connection.
   //

   unsigned long          OpenFiles;

   //
   // Maximum number of concurrent open files
   // at any one time on this connection.
   //

   unsigned long          MaxOpenFiles;

   //
   // Current number of connection handles
   // opened on this connection reference
   //

   unsigned long          CurrentConnectionHandles;

   //
   // Maximum number of connection handles
   // opened at a time on this connection
   // reference.
   //

   unsigned long          MaxConnectionHandles;

   //
   // Current license count on this connection
   // reference.  (Number of handles that have
   // licensed reference)
   //

   unsigned long          CurrentLicenseCount;

   //
   // The maximum license count on this reference
   // at any one time.
   //

   unsigned long          MaxLicenseCount;

   //
   // The total number of bytes read on this connection
   //

   unsigned long long  TotalBytesRead;

   //
   // The total number of bytes written on this connection
   //

   unsigned long long  TotalBytesWritten;

   //
   // The total number of I/O requests completed
   // on this connection reference.
   //

   unsigned long          TotalIoRequestsCompleted;

   //
   // The total number of I/O requests that have
   // been queued to a worker thread
   //

   unsigned long          TotalAsynchronousIoRequests;

   //
   // The total number of NCPs completed on this
   // connection reference
   //

   unsigned long          TotalNcpsCompleted;

   //
   // Current number of tasks allocated to
   // this connection reference.
   //

   unsigned long          CurrentTasksInUse;

   //
   // Maximum number of tasks allocated
   // to this reference at any one time.
   //

   unsigned long          MaxTasksInUse;

   //
   // Total number of LIP negotiations
   // on this connection reference.
   //

   unsigned long          LipNegotiations;

   //
   // TRUE if a packet burst connection is
   // active on this reference.
   //

   unsigned short int    PacketBurstConnection;

   //
   // TRUE if the connection reference has
   // been flagged permanent.
   //

   unsigned short int	  ConnectionPermanent;

   //
   // The following fields are statistics which
   // are bound to the connection handle.
   //

   //
   // Task number assigned to this connection handle.
   //

   unsigned long          TaskNumber;

   //
   // Total number of times this connection handle has
   // been accessed.
   //

   unsigned long          TotalHandleAccesses;

} ConnStatistics, *PConnStatistics;


//
// General Redirector statistics
//

typedef  struct   tagRedirStatistics
{
   //
   // Number of Ioctls issued to the NetWare redirector.
   //

   unsigned long          RedirIoctls;

   //
   // Number of requests queued to the FSP overflow queue
   //

   unsigned long          RequestsQueuedToOverflow;

   //
   // Number network I/O contexts allocated from a zone
   //

   unsigned long          IoContextZoneHits;


   //
   // Number network I/O contexts allocated where the zone
   // was depleted and the request came from non-paged memory
   //

   unsigned long          IoContextZoneMisses;

} RedirStatistics, *PRedirStatistics;


//
// Get Browse Connection References
//

typedef struct _GetBrowseConnectionsRec {

   unsigned long    recordSize;
   unsigned long    numConnectionsReturned;
   unsigned long    numConnectionsAvailable;
   unsigned long    connReferences[1];

} GetBrowseConnectionRec, *PGetBrowseConnectionRec;




//
// DeviceIoControl requests for the NetWare Redirector
//
// Macro definition for defining DeviceIoControl function control codes.
// The function codes 0 - 2047 are reserved for Microsoft.
// Function codes 2048 - 4096 are reserved for customers.
// The NetWare Redirector will use codes beginning at 3600.
//
// METHOD_NEITHER User buffers will be passed directly from the application
// to the file system.  The redirector is responsible for either probing
// and locking the buffers or using a try - except around access of the
// buffers.

#define  BASE_REQ_NUM                  0x4a541000

// Connection functions
#define  NWC_OPEN_CONN_BY_NAME         (BASE_REQ_NUM + 0)
#define  NWC_OPEN_CONN_BY_ADDRESS      (BASE_REQ_NUM + 1)
#define  NWC_OPEN_CONN_BY_REFERENCE    (BASE_REQ_NUM + 2)
#define  NWC_CLOSE_CONN                (BASE_REQ_NUM + 3)
#define  NWC_SYS_CLOSE_CONN            (BASE_REQ_NUM + 4)
#define  NWC_GET_CONN_INFO             (BASE_REQ_NUM + 5)
#define  NWC_SET_CONN_INFO             (BASE_REQ_NUM + 6)
#define  NWC_SCAN_CONN_INFO            (BASE_REQ_NUM + 7)
#define  NWC_MAKE_CONN_PERMANENT       (BASE_REQ_NUM + 8)
#define  NWC_LICENSE_CONN              (BASE_REQ_NUM + 9)
#define  NWC_UNLICENSE_CONN            (BASE_REQ_NUM + 10)
#define  NWC_GET_NUM_CONNS             (BASE_REQ_NUM + 11)
#define  NWC_GET_PREFERRED_SERVER      (BASE_REQ_NUM + 12)
#define  NWC_SET_PREFERRED_SERVER      (BASE_REQ_NUM + 13)
#define  NWC_GET_PRIMARY_CONN          (BASE_REQ_NUM + 14)
#define  NWC_SET_PRIMARY_CONN          (BASE_REQ_NUM + 15)

// Authentication functions
#define  NWC_CHANGE_KEY                (BASE_REQ_NUM + 20)
#define  NWC_ENUMERATE_IDENTITIES      (BASE_REQ_NUM + 21)
#define  NWC_GET_IDENTITY_INFO         (BASE_REQ_NUM + 22)
#define  NWC_LOGIN_IDENTITY            (BASE_REQ_NUM + 23)
#define  NWC_LOGOUT_IDENTITY           (BASE_REQ_NUM + 24)
#define  NWC_SET_KEY                   (BASE_REQ_NUM + 25)
#define  NWC_VERIFY_KEY                (BASE_REQ_NUM + 26)
#define  NWC_AUTHENTICATE_CONN_WITH_ID (BASE_REQ_NUM + 27)
#define  NWC_UNAUTHENTICATE_CONN       (BASE_REQ_NUM + 28)

// Directory Services functions
#define  NWC_GET_DEFAULT_NAME_CONTEXT  (BASE_REQ_NUM + 30)
#define  NWC_SET_DEFAULT_NAME_CONTEXT  (BASE_REQ_NUM + 31)
#define  NWC_GET_PREFERRED_DS_TREE     (BASE_REQ_NUM + 32)
#define  NWC_SET_PREFERRED_DS_TREE     (BASE_REQ_NUM + 33)
#define  NWC_GET_TREE_MONITORED_CONN_REF  (BASE_REQ_NUM + 34)
#define  NWC_NDS_RESOLVE_NAME_TO_ID    (BASE_REQ_NUM + 35)

// NCP Request functions
#define  NWC_FRAGMENT_REQUEST          (BASE_REQ_NUM + 40)
#define  NWC_NCP_ORDERED_REQUEST_ALL   (BASE_REQ_NUM + 41)
#define  NWC_RAW_NCP_REQUEST           (BASE_REQ_NUM + 42)
#define  NWC_RAW_NCP_REQUEST_ALL       (BASE_REQ_NUM + 43)

// File Handle Conversion functions
#define  NWC_CONVERT_LOCAL_HANDLE      (BASE_REQ_NUM + 50)
#define  NWC_CONVERT_NETWARE_HANDLE    (BASE_REQ_NUM + 51)

// Misc. functions
#define  NWC_MAP_DRIVE                 (BASE_REQ_NUM + 60)
#define  NWC_UNMAP_DRIVE               (BASE_REQ_NUM + 61)
#define  NWC_ENUMERATE_DRIVES          (BASE_REQ_NUM + 62)

#define  NWC_GET_REQUESTER_VERSION     (BASE_REQ_NUM + 63)
#define  NWC_QUERY_FEATURE             (BASE_REQ_NUM + 64)

#define  NWC_GET_CONFIGURED_NSPS       (BASE_REQ_NUM + 65)

#define  NWC_GET_MOUNT_PATH            (BASE_REQ_NUM + 66)

#define  NWC_GET_BROADCAST_MESSAGE     (BASE_REQ_NUM + 67)

//
// NetWare Redirector for Windows NT NWC API Set
//

//++=======================================================================
//  API Name:        NwcAllocTdsNode
//
//  Arguments In:    uTag - Unique Id of the tag being allocated.
//
//                   uNodeSize - Maximum amount of data this node can
//                   store.
//
//                   uTdsFlags - Flags describing actions to be taken
//                   on the node. (ex. encrypt, post-zero, pre-zero)
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_INSUFFICIENT_RESOURCES
//                   NWE_PARAM_INVALID
//                   NWE_TDS_TAG_IN_USE
//
//  Abstract:        This API allocates a tagged data store node and
//                   associates the unique caller assigned tag to
//                   the new node.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcAllocTdsNode
{
       unsigned long                uTag;
       unsigned long                uNodeSize;
       unsigned long                uTdsFlags;

} NwcAllocTdsNode, *PNwcAllocTdsNode;

//++=======================================================================
//  API Name:        NwcAuthenticate
//
//  Arguments In:    ConnHandle - The connection to be authenticated
//
//                   uAuthenticationType - This is the type of
//                   authentication that will be used with this information
//                   such as DS, BINDERY or PNW
//
//                   pUserName - The name of the user that is being
//                   authenticated with this Id.
//
//                   pPassword - The clear text password to be used in
//                   authenticating with the returned Id.
//
//                   MaximumLength -
//
//                   InfoLength - Length of the following authentication
//                   information data.
//
//                   pAuthenticationInfo - Specific authentication data
//                   for each different authentication package.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_INSUFFICIENT_RESOURCES
//
//  Abstract:        This API is used to authenticate a connection without
//                   creating an authentication Id first.  It requires that
//                   all of hte information that is needed to authenticate
//                   a connection must be explicitly passed in.
//
//  Notes:           ONLY HERE FOR LIBRARY SUPPORT
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcAuthenticate
{
       NW_CONN_HANDLE       ConnHandle;
       unsigned long        uAuthenticationType;
       PCHAR                pUserName;
       PCHAR                pPassword;
       unsigned long        MaxInfoLength;
       unsigned long        InfoLength;
       void					*pAuthenInfo;

} NwcAuthenticate, *PNwcAuthenticate;

//++=======================================================================
//  API Name:        NwcCancelCapture
//
//  Arguments In:    uLptDevice - The number of the LPT device to stop
//                   redirecting.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_DEVICE_NOT_REDIRECTED
//
//  Abstract:        This API ends the redirection of the specified LPT
//                   device.  If the specified LPT device was redirecting
//                   data to a capture file, both the capture file and the
//                   associated print queue job entry are deleted.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcCancelCapture
{
       unsigned long                uLptDevice;

} NwcCancelCapture, *PNwcCancelCapture;

//++=======================================================================
//  API Name:        NwcCheckOpenFileOnReference
//
//  Arguments In:    uConnReference - The connection reference to check
//                   for opened files.
//
//  Arguments Out:   fbOpenFiles    TRUE if files are open on the
//                                  connection reference.
//
//                                  FALSE if no files are open on the
//                                  connection reference.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API checks to see if any open files exist
//                   on the specified connection reference.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcCheckOpenFilesOnRef
{
      unsigned long                uConnReference;
      unsigned short int              fbOpenFiles;

} NwcCheckOpenFilesOnRef, *PNwcCheckOpenFilesOnRef;

//++=======================================================================
//  API Name:        NwcClearBroadcastMessage
//
//  Arguments In:    NONE
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//
//  Abstract:        This API is clears the broadcast message buffer.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--


//++=======================================================================
//  API Name:        NwcCloseConn
//
//  Arguments In:    ConnHandle - The handle to a connection that is
//                   no longer needed.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//                   NWE_INVALID_OWNER
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API is used by an application that opened the
//                   connection using one of the open connection calls
//                   is finished using the connection.  After it is closed,
//                   the handle may no longer be used to access the
//                   connection.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcCloseConn
{
       NW_CONN_HANDLE       ConnHandle;

} NwcCloseConn, *PNwcCloseConn;

//++=======================================================================
//  API Name:        NwcConvertLocalFileHandle
//
//  Arguments In:    NONE
//
//  Arguments Out:   uConnReference - The connection reference associated
//                   with the returned NetWare file handle.
//
//                   pNetWareFileHandle - The six byte NetWare file handle
//                   associated with the given local file handle.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_RESOURCE_NOT_OWNED
//
//  Abstract:        This API is used to return the NetWare handle that
//                   has been associated to a local file handle.
//                   In addition to returning the NetWare file handle,
//                   this API also returns the connection reference to
//                   the connection that owns the file.
//
//  Notes:           This API does not create a new NetWare handle, it
//                   only returns the existing handle associated to the
//                   local handle.
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcConvertLocalHandle
{
      unsigned long     uConnReference;
      unsigned char     NwFileHandle[6];

} NwcConvertLocalHandle, *PNwcConvertLocalHandle;

//++=======================================================================
//  API Name:        NwcConvertNetWareHandle
//
//  Arguments In:    ConnHandle - The connection associated with the
//                   NetWare file handle to convert.
//
//                   uAccessMode - The access rights to be used when
//                   allocating the local file handle.
//
//                   pNetWareHandle - The NetWare handle that will be
//                   bound to the new local handle being created.
//
//                   uFileSize - The current file size of the NetWare
//                   file associated with the given NetWare file handle.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_RESOURCE_NOT_OWNED
//
//  Abstract:        This API is used to convert a NetWare file handle
//                   to a local file handle.
//
//                   The local handle must have been created previously
//                   by doing a local open to \Special\$Special.net.
//
//                   Then an Ioctl to this function must be issued using the
//                   handle returned from the special net open.
//
//  Notes:           After making this call, the NetWare file handle
//                   should not be closed using the NetWare library
//                   call, instead it should be closed using the local
//                   operating system's close call.
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcConvertNetWareHandle
{
          NW_CONN_HANDLE       ConnHandle;
          unsigned long        uAccessMode;
          unsigned char        NetWareHandle[6];
          unsigned long        uFileSize;

} NwcConvertNetWareHandle, *PNwcConvertNetWareHandle;

//++=======================================================================
//  API Name:        NwcEndCapture
//
//  Arguments In:    uLptDevice - The number of the LPT device to stop
//                   redirecting.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_DEVICE_NOT_REDIRECTED
//
//  Abstract:        This API ends the redirection of the specified LPT
//                   device.  If the specified LPT device was redirecting
//                   data to a capture file, the file is closed and the
//                   print queue is allowed to finish.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcEndCapture
{
     unsigned long                uLptDevice;

} NwcEndCapture, *PNwcEndCapture;

//++=======================================================================
//  API Name:        NwcFlushCapture
//
//  Arguments In:    uLptDevice - The number of the LPT device that is
//                   to be flushed.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_DEVICE_NOT_REDIRECTED
//
//  Abstract:        This API causes the specified Lpt device to continue
//                   to be redirected but when more data is written to
//                   the device a new print job file is created for that
//                   data.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcFlushCapture
{
    unsigned long                uLptDevice;

} NwcFlushCapture, *PNwcFlushCapture;

//++=======================================================================
//  API Name:        NwcFragmentRequest
//
//  Arguments In:    ConnHandle
//                      The connection handle the request is being
//                      directed to.
//
//                   uFunction
//                      The NCP function to be called, should be 104
//                      for NDS fragger/defragger requests.
//
//                   uSubFunction
//                      The NCP subfunction to be called, should be
//                      2 for NDS fragger/defragger requests.
//
//                   uVerb
//                      The actual operation to be completed on the
//                      server backend.
//
//                   flags
//                      Currently not implemented.  Reserved for
//                      future use.
//
//                   uNumRequestFrags
//                      The number of fragments that the request packet
//                      has been broken into.
//
//                   pRequestFrags
//                      List of fragments that make up the request packet.
//                      Each fragment includes the length of the fragment
//                      data and a pointer to the data.
//
//                   uNumReplyFrags
//                      The number of fragments the reply packet has been
//                      broken into.
//
//  Arguments Out:   pReplyFrags
//                      List of fragments that make up the reply packet.
//                      Each fragment includes the length of the fragment
//                      data and a pointer to the data.
//
//                   uActualReplyLength
//                      Total size of the reply packet after any header
//                      and tail information is removed.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//
//  Abstract:        API for sending large NCP/NDS packets that are
//                   larger than the max MTU size for the underlying
//                   network.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcFragmentRequest
{
         NW_CONN_HANDLE       ConnHandle;
         unsigned long        uFunction;
         unsigned long        uSubFunction;
         unsigned long        uVerb;
         unsigned long        flags;
         unsigned long        uNumRequestFrags;
         PNwcFrag             pRequestFrags;
         unsigned long        uNumReplyFrags;
         PNwcFrag             pReplyFrags;
         unsigned long        uActualReplyLength;

} NwcFragmentRequest, *PNwcFragmentRequest;

//++=======================================================================
//  API Name:        NwcFreeTdsNode
//
//  Arguments In:    uTag - Unique Id of the tag created previously.
//
//  Arguments Out:   uMaxSize - Maximum data size available for this
//                   TDS node.
//
//                   uDataSize - Amount of data in bytes of the tag
//                   that is currently being used.
//
//                   uTdsFlags - Flags that were used when the TDS node
//                   was created.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_TDS_VALID_TAG
//
//  Abstract:        This API frees a previously allocated TDS node.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcFreeTdsNode
{
     unsigned long                uTag;

} NwcFreeTdsNode, *PNwcFreeTdsNode;

//++=======================================================================
//  API Name:        NwcGetBannerUserName
//
//  Arguments In:    NONE
//
//  Arguments Out:   pBannerUserName - The global banner user name that is
//                   sent with the print jobs from this workstation.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_INVALID_PARAM
//                   NWE_OBJECT_NOT_FOUND
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API is returns the global banner user name for
//                   this system that will be sent to the print queue
//                   for every job printed where banners are turned on.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetBannerUserName
{
   unsigned char *pBannerUserName;

} NwcGetBannerUserName, *PNwcGetBannerUserName;

//++=======================================================================
//  API Name:        NwcGetBroadcastMessage
//
//  Arguments In:    uMessageFlags - Flag describing if the caller wants
//                   to find the next connection reference that has a
//                   message waiting or if the caller wants to wait
//                   until he is notified of a pending message on a
//                   connection.
//
//  Arguments Out:   uConnReference - The connection reference of the
//                   pending message.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_NO_MORE_ENTRIES
//                   NWE_REDIRECTOR_ERROR
//
//  Abstract:        This API is used for notifying a caller of pending
//                   broadcast messages on the server.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetBroadcastNotification
{
        unsigned long       uMessageFlags;
        unsigned long       uConnReference;
        unsigned long       messageLen;
        char                message[1];

} NwcGetBroadcastNotification, *PNwcGetBroadcastNotification;

//++=======================================================================
//  API Name:        NwcGetConnInfo
//
//  Arguments In:    ConnHandle - Connection handle for the connection to
//                   get information on.
//                   uInfoLevel - Specifies what information should be
//                   returned.
//                   uInfoLen - Length of the ConnInfo buffer.
//
//  Arguments Out:   pConnInfo - A pointer to a buffer to return connection
//                   information in.  If the caller is requesting all
//                   information the pointer will be to a structure of
//                   type NwcConnInfo.  If the caller is requesting just
//                   a single piece of information, the pointer is the
//                   type of information being requested.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//                   NWE_INVALID_OWNER
//                   NWE_RESOURCE_LOCK
//                   NWE_STRING_TRANSLATION
//
//  Abstract:        This API returns connection information for the specified
//                   connection.  The requester can receive one piece of
//                   information or the whole information structure.
//                   Some of the entries in the NwcConnInfo structure are
//                   pointers.  The requester is responsible for supplying
//                   valid pointers for any info specified to be returned.
//                   If the requester does not want a piece of information
//                   returned, a NULL pointer should be placed in the field.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetConnInfo
{
         NW_CONN_HANDLE       ConnHandle;
         unsigned long        uInfoLevel;
         unsigned long        uInfoLength;
         void				  *pConnInfo;

} NwcGetConnInfo, *PNwcGetConnInfo;

//++=======================================================================
//  API Name:        NwcGetConnStats
//
//  Arguments In:    ConnHandle - Open handle to the connection to obtain
//                   statistics for.
//
//  Arguments Out:   pConnStats - Pointer to a ConnStatistics structure.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API returns statistics about the current state
//                   and past history of the connection.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetConnStats
{
         NW_CONN_HANDLE       ConnHandle;
         PConnStatistics      pConnStats;

} NwcGetConnStats, *PNwcGetConnStats;

//++=======================================================================
//  API Name:        NwcGetConnTableStats
//
//  Arguments In:    NONE
//
//  Arguments Out:   pConnTableStats - Pointer
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API returns statistics about the history
//                   and current state of the connection table.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetConnTableStats
{
        PConnTableStatistics pConnTableStats;

} NwcGetConnTableStats, *PNwcGetConnTableStats;

//++=======================================================================
//  API Name:        NwcGetDefaultNameContext
//
//  Arguments In:    pNameLength - On input, this is the length of the
//                   name context buffer.  On output, this is the actual
//                   length of the name context string.
//
//  Arguments Out:   pNameContext - The buffer to copy the default name
//                   context into.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_BUFFER_OVERFLOW
//                   NWE_OBJECT_NOT_FOUND
//                   NWE_PARAM_INVALID
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API returns the default name context that
//                   was previously set either by configuration or
//                   by calling NwcSetDefaultNameContext.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetDefaultNameContext
{
       unsigned long        uTreeLength;
       unsigned char		*pTreeName;
       unsigned long        uNameLength;
       unsigned short int	*pNameContext;

} NwcGetDefaultNameContext, *PNwcGetDefaultNameContext;

//++=======================================================================
//  API Name:        NwcGetGlobalEvent
//
//  Arguments In:    pEventPacket - On input, the event registration
//                   field needs to be filled in with the event type
//                   or events the caller wishes to be notified for.
//
//  Arguments Out:   pEventPacket - On output, the event received field
//                   specifies the type of event received the data
//                   field describes the event.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_BUFFER_OVERFLOW
//                   NWE_PARAM_INVALID
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API is used to receive global events about
//                   changes happening in the redirector.  Such as
//                   broadcast notification, connection creation/term,
//                   drive mappings etc.
//
//                   The caller must issue this request then block either
//                   explicitly on an event or implicitly by the type
//                   of device open.
//
//                   The caller may also OR the event types to receive
//                   notification for different types of events occurring
//                   in the system.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetGlobalEvent
{
      PGlobalEvent         pEventPacket;

} NwcGetGlobalEvent, *PNwcGetGlobalEvent;

//++=======================================================================
//  API Name:        NwcGetMapInformation
//
//  Arguments In:    uDriveNumber - The drive number for which to obtain
//                   status (Ex. A=1, B=2, C=3 ...).
//
//  Arguments Out:   uMapStatus - A bit mask indicating if the drive is
//                   local and/or remote.
//
//                   uConnReference - A connection reference to the
//                   connection to which the drive is mapped.
//
//                   uDirHandle - The directory handle associated with
//                   pDirPath that defines the directory path of the
//                   specified drive.
//
//                   pDirPath - The directory path relative to uDirHandle
//                   that defines where the drive is mapped.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_INVALID_DRIVE_NUMBER
//                   NWE_STRING_TRANSLATION
//
//  Abstract:        This API returns the status, connection and directory
//                   path associated with the specified drive.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetMapInfo
{
        unsigned long       uDriveNumber;
        unsigned long       uMapStatus;
        unsigned long       uConnReference;
        unsigned long       uDirHandle;
        char 				*pDirPath;

} NwcGetMapInfo, *PNwcGetMapInfo;

//++=======================================================================
//  API Name:        NwcGetMaxPrinters
//
//  Arguments In:    NONE
//
//  Arguments Out:   uMaxPrinters - The maximum number of LPT ports that
//                   can be redirected.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:        This API returns the maximum number of LPT ports
//                   that can be redirected (captured) on this platform.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetMaxPrinters
{
       unsigned long                uMaxPrinters;

} NwcGetMaxPrinters, *PNwcGetMaxPrinters;

//++=======================================================================
//  API Name:        NwcGetTreeMonitoredConnReference
//
//  Arguments In:    NONE
//
//  Arguments Out:   uConnReference - The connection reference associated
//                   with the monitored connection.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_OBJECT_NOT_FOUND
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This call returns a connection reference to a
//                   connection that is monitored.  This connection
//                   reference may be used to open the connection.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetTreeMonitoredConnRef
{
      PNwcString        pTreeName;
      unsigned long             uConnReference;

} NwcGetTreeMonitoredConnRef, *PNwcGetTreeMonitoredConnRef;

//++=======================================================================
//  API Name:        NwcGetMonitoredConnReference
//
//  Arguments In:    NONE
//
//  Arguments Out:   uConnReference - The connection reference associated
//                   with the monitored connection.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_OBJECT_NOT_FOUND
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This call returns a connection reference to a
//                   connection that is monitored.  This connection
//                   reference may be used to open the connection.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetMonitoredConnRef
{
       unsigned long             uConnReference;

} NwcGetMonitoredConnRef, *PNwcGetMonitoredConnRef;

//++=======================================================================
//  API Name:        NwcGetNumberConns
//
//  Arguments In:    NONE
//
//  Arguments Out:   uMaxConns - The maximum number of connections
//                   supported by the redirector.  -1 for dynamic.
//
//                   uPublicConns - The current number of public
//                   connections.
//
//                   uTasksPrivateConns - The current number of private
//                   connections that are owned by the calling process.
//
//                   uOtherPrivateConns - The current number of private
//                   connections that are not owned by the calling
//                   process.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API returns the current number of connections
//                   as well as the maximum number of supported
//                   connections.  If the requester/redirector supports
//                   a dynamic connection table, -1 will be returned
//                   in the uMaxConns field.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetNumberConns
{
        unsigned long                uMaxConns;
        unsigned long                uPublicConns;
        unsigned long                uTasksPrivateConns;
        unsigned long                uOtherPrivateConns;

} NwcGetNumberConns, *PNwcGetNumberConns;

//++=======================================================================
//  API Name:        NwcGetPreferredServer
//
//  Arguments In:    uServerNameLength - On input, this is the length
//                   in bytes of the server buffer.  On output, this is
//                   the actual length of the server name string in bytes.
//
//  Arguments Out:   pServerName - The buffer to copy the preferred server
//                   name into.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_BUFFER_OVERFLOW
//                   NWE_OBJECT_NOT_FOUND
//                   NWE_PARAM_INVALID
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API returns the configured preferred bindery
//                   server previously set either by configuration or
//                   by calling NwcSetPreferredServer.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetPreferredServer
{
      unsigned long     uServerNameLength;
      char				*pServerName;

} NwcGetPreferredServer, *PNwcGetPreferredServer;

//++=======================================================================
//  API Name:        NwcGetPreferredDsTree
//
//  Arguments In:    uTreeLength - On input, this is the length in bytes
//                   of the DS tree name buffer.  On output, this is the
//                   actual length of the DS tree name string in bytes.
//
//  Arguments Out:   pDsTreeName - The buffer to copy the DS tree name into.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_BUFFER_OVERFLOW
//                   NWE_PARAM_INVALID
//                   NWE_DS_PREFERRED_NOT_FOUND
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API returns the preferred DS tree name that was
//                   previously set either by configuration or
//                   by calling NwcSetPreferredDsTree.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetPreferredDsTree
{
      unsigned long       uTreeLength;
      unsigned char       *pDsTreeName;

} NwcGetPreferredDsTree, *PNwcGetPreferredDsTree;

//++=======================================================================
//  API Name:        NwcGetPrimaryConnection
//
//  Arguments In:    NONE
//
//  Arguments Out:   uConnReference - Reference to the primary connection.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_PRIMARY_NOT_SET
//
//  Abstract:        This API returns the reference to the current primary
//                   connection in the redirector.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetPrimaryConnection
{
       unsigned long                uConnReference;

} NwcGetPrimaryConnection, *PNwcGetPrimaryConnection;

//++=======================================================================
//  API Name:        NwcGetPrinterStrings
//
//  Arguments In:    uLptDevice - The redirected LPT device to get the
//                   printer setup strings for.
//
//  Arguments Out:   uSetupStringLength - The length of the setup string
//                   buffer on input and the actual length of the setup
//                   string on output.
//
//                   pSetupString - The string that is sent to the print
//                   queue on this redirected device at the time the
//                   Lpt device is opened.  This field can be NULL if
//                   the caller does not want to return the string.
//
//                   uResetStringLength - The length of the reset string
//                   buffer on input and the actual length of the reset
//                   string on output.
//
//                   pResetString - The string that is sent to the print
//                   queue at the end of a print job.  This field can be
//                   NULL if the caller does not want to return the
//                   string.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_BUFFER_OVERFLOW
//                   NWE_DEVICE_NOT_REDIRECTED
//
//  Abstract:        This API is used to get the printer setup and reset
//                   strings for the given redirected device.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetPrinterStrings
{
      unsigned long         uLptDevice;
      unsigned long         uSetupStringLength;
      unsigned char			*pSetupString;
      unsigned long         uResetStringLength;
      unsigned char         *pResetString;

} NwcGetPrinterStrings, *PNwcGetPrinterStrings;

//++=======================================================================
//  API Name:        NwcGetRedirectorSystemStats
//
//  Arguments In:    NONE
//
//  Arguments Out:   pSysStats - Pointer to a RedirSystemStats structure.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:        This API returns system statistics about the
//                   redirector.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetRedirectorSytsemStats
{
       PRedirStatistics     pRedirSystemStats;

} NwcGetRedirectorSystemStats, *PNwcGetRedirectorSystemStats;

//++=======================================================================
//  API Name:        NwcGetRequesterVersion
//
//  Arguments In:    NONE
//
//  Arguments Out:   uMajorVersion
//                   uMinorVersion
//                   uRevision
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:        This API returns the major version, minor version and
//                   revision of the requester/redirector.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetRequesterVersion
{
         unsigned long                uMajorVersion;
         unsigned long                uMinorVersion;
         unsigned long                uRevision;

} NwcGetRequesterVersion, *PNwcGetRequesterVersion;

//++=======================================================================
//  API Name:        NwcGetTdsInfo
//
//  Arguments In:    uTag - Unique Id of the tag created previously.
//
//  Arguments Out:   uMaxSize - Maximum data size available for this
//                   TDS node.
//
//                   uDataSize - Amount of data in bytes of the tag
//                   that is currently being used.
//
//                   uTdsFlags - Flags that were used when the TDS node
//                   was created.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_TDS_INVALID_TAG
//
//  Abstract:        This API is returns information about a previously
//                   allocated TDS node.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetTdsInfo
{
         unsigned long                uTag;
         unsigned long                uMaxSize;
         unsigned long                uDataSize;
         unsigned long                uTdsFlags;

} NwcGetTdsInfo, *PNwcGetTdsInfo;

//++=======================================================================
//  API Name:        NwcLicenseConn
//
//  Arguments In:    ConnHandle - An open connection handle that is in
//                   an unlicensed state.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//                   NWE_HANDLE_ALREADY_LICENSED
//
//
//  Abstract:        This API changes a connections state to licensed.
//                   The licensed count will be incremented, and if
//                   necessary, the license NCP will be sent.
//                   If this handle is already in a licensed state,
//                   an error will be returned.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcLicenseConn
{
          NW_CONN_HANDLE       ConnHandle;

} NwcLicenseConn, *PNwcLicenseConn;

//++=======================================================================
//  API Name:        NwcMakeConnPermanent
//
//  Arguments In:    ConnHandle - An open connection handle associated
//                   with the connection to be made permanent.
//
//  Arguments Out:   NONE
//
//  Returns:         NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//                   NWE_INVALID_OWNER
//
//  Abstract:        This API is used to keep the connection from being
//                   destroyed until a NwcSysCloseConn request is made
//                   on the connection.  This allows the connection to
//                   remain after all processes that have the
//                   connection open terminate.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcMakeConnPermanent
{
          NW_CONN_HANDLE       ConnHandle;

} NwcMakeConnPermanent, *PNwcMakeConnPermanent;

//++=======================================================================
//  API Name:        NwcMapDrive
//
//  Arguments In:    ConnHandle - The connection handle of the server
//                   to where the drive is to be mapped.
//
//                   uDriveNumber - The number of the drive to be
//                   mapped (A=1, B=2, C=3 ...)
//
//                   uDirHandle - The directory handle associated
//                   with the pDirPath.
//
//                   pDirPath - The directory path that the drive will
//                   be mapped to.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//                   NWE_INSUFFICIENT_RESOURCES
//                   NWE_INVALID_DRIVE_NUMBER
//                   NWE_STRING_TRANSLATION
//
//  Abstract:        This API maps the target drive to the specified
//                   directory.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcMapDrive
{
          NW_CONN_HANDLE       ConnHandle;
          unsigned long                uDriveNumber;
          unsigned long                uDirHandle;
          PCHAR                pDirPath;

} NwcMapDrive, *PNwcMapDrive;


typedef  struct   tagNwcMapDriveEx
{
          NW_CONN_HANDLE        ConnHandle;
          unsigned long         localUid;
          unsigned long         linkOffsetLength;
          unsigned long         linkOffset;
          unsigned long         dirPathOffsetLength;
          unsigned long         dirPathOffset;

} NwcMapDriveEx, *PNwcMapDriveEx;


//++=======================================================================
//  API Name:        NWCGetMappedDrives
//
//  Arguments In:    
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_BUFFER_OVERFLOW
//
//  Abstract:        This API returns the NetWare mapped drive info
//                   per user.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, LINUX
//
//=======================================================================--

typedef  struct   tagNwcMapDriveElem
{
   unsigned long    ElemLen;          // Lenght of drive element
   unsigned long    ConnRefernce;     // Connection reference
   unsigned long    LinkPathLen;      // Local/link dir path, length includes nul
   unsigned char    LinkPath[1];      // LinkPath[LocalDirPathLen]

//   OUT      unsigned long                DirPathLen;       // NetWare dir path, length includes nul (vol:path)
//   OUT      UCHAR                DirPath[1];       // NetWarePath[NetWarePathLen]

} NwcMapDriveElem, *PNwcMapDriveElem;

//#define MAP_DRIVE_ELEM_SIZE(LocalPathLen, NWPathLen)  (sizeof(MapDriveElem) + LocalPathLen + NWPathLen - 2)

typedef  struct   tagNwcMapDriveBuff
{
   unsigned long     MapCount;         // Number of mapped drives
   NwcMapDriveElem   MapDriveElem[1];  // MapDriveElem[MapCount]

} NwcMapDriveBuff, *PNwcMapDriveBuff;

typedef  struct   tagNwcGetMappedDrives
{
   unsigned long                MapBuffLen;       // Buffer length (actual buffer size returned)
   PNwcMapDriveBuff     MapBuffer;        // Number of mapped drives

} NwcGetMappedDrives, *PNwcGetMappedDrives;


//++=======================================================================
//  API Name:        NwcGetMountPath
//
//  Arguments In:    MountPathLen - Length of mount path buffer
//                   including nul terminator.
//
//  Arguments Out:   MountPath - Pointer to mount path buffer
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_BUFFER_OVERFLOW
//
//  Abstract:        This API returns the mount point of the NOVFS file
//                   system.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, LINUX
//
//=======================================================================--

typedef  struct   tagNwcGetMountPath
{
   unsigned long        MountPathLen;
   unsigned char       *pMountPath;

} NwcGetMountPath, *PNwcGetMountPath;


//++=======================================================================
//  API Name:        NwcMonitorConn
//
//  Arguments In:    ConnHandle - The handle associated with the connection
//                   that is to be marked as the monitored connection.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_RESOURCE_LOCK
//                   NWE_CONN_INVALID
//
//
//  Abstract:        This call marks the connection associated with the
//                   connection handle as monitored.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcMonitorConn
{
          NW_CONN_HANDLE       ConnHandle;

} NwcMonitorConn, *PNwcMonitorConn;

//++=======================================================================
//  API Name:        NwcOpenConnByAddr
//
//  Arguments In:    pServiceType - The type of service required.
//
//                   uConnFlags - Specifies whether this connection
//                   should be public or private.
//
//                   pTranAddress - Specifies the transport address of
//                   the service to open a connection on.
//                   a connection to.
//
//  Arguments Out:   ConnHandle - The new connection handle returned.
//                   This handle may in turn be used for all requests
//                   directed to this connection.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_INSUFFICIENT_RESOURCES
//                   NWE_TRAN_INVALID_TYPE
//                   NWE_RESOURCE_LOCK
//                   NWE_UNSUPPORTED_TRAN_TYPE
//
//  Abstract:        This API will create a service connection to
//                   the service specified by the transport address.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcOpenConnByAddr
{
         char			 *pServiceType;
         unsigned long   uConnFlags;
         PNwcTranAddr    pTranAddr;
         NW_CONN_HANDLE  ConnHandle;

} NwcOpenConnByAddr, *PNwcOpenConnByAddr;

//++=======================================================================
//  API Name:        NwcOpenConnByName
//
//  Arguments In:    ConnHandle - The connection to use when resolving
//                   a name.  For instance, if the name is a bindery name
//                   the requester will scan the bindery of the given
//                   connection to retrieve the service's address.  This
//                   value can also be NULL if the caller doesn't care
//                   which connection is used to resolve the address.
//
//                   pName - A pointer to the name of the service trying
//                   to be connected to.  This string is NULL terminated,
//                   contains no wild cards, and is a maximum of 512
//                   characters long.
//
//                   pServiceType - The type of service required.
//
//                   uConnFlags - Specifies whether this connection
//                   should be public or private.
//
//                   uTranType - Specifies the preferred or required
//                   transport type to be used.
//                   NWC_TRAN_TYPE_WILD may be ORed with the other values
//                   or used alone.  When ORed with another value, the
//                   wild value indicates an unmarked alternative is
//                   acceptable.  When used alone, the current preferred
//                   transport is used.
//
//  Arguments Out:   ConnHandle - The new connection handle returned.
//                   This handle may in turn be used for all requests
//                   directed to this connection.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_BUFFER_OVERFLOW
//                   NWE_INSUFFICIENT_RESOURCES
//                   NWE_INVALID_STRING_TYPE
//                   NWE_RESOURCE_LOCK
//                   NWE_STRING_TRANSLATION
//                   NWE_TRAN_INVALID_TYPE
//                   NWE_UNSUPPORTED_TRAN_TYPE
//
//  Abstract:        This API will resolve the given name to a network
//                   address then create a service connection to the
//                   specified service.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcOpenConnByName
{
         NW_CONN_HANDLE     ConnHandle;
         PNwcConnString     pName;
         char 				*pServiceType;
         unsigned long      uConnFlags;
         unsigned long      uTranType;
         NW_CONN_HANDLE     RetConnHandle;

} NwcOpenConnByName, *PNwcOpenConnByName;

//++=======================================================================
//  API Name:        NwcOpenConnByReference
//
//  Arguments In:    uConnReference - A reference handle which identifies
//                   a valid connection that the caller wants to obtain
//                   a connection handle to.  A reference handle can be
//                   used to get information about the connection without
//                   actually getting a handle to it.  A connection handle
//                   must be used to make actual requests to that
//                   connection.
//
//                   uConnFlags - Currently unused.
//
//  Arguments Out:   ConnHandle - The new connection handle returned.
//                   This handle may in turn be used for all requests
//                   directed to this connection.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//
//  Abstract:        This API will open the connection associated with
//                   the given connection reference.  The connection
//                   reference can be obtained by calling the
//                   NwcScanConnInfo API.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcOpenConnByReference
{
         unsigned long                uConnReference;
         unsigned long                uConnFlags;
         NW_CONN_HANDLE       ConnHandle;

} NwcOpenConnByReference, *PNwcOpenConnByReference;

//++=======================================================================
//  API Name:        NwcReadTds
//
//  Arguments In:    uTag - Unique Id of the tag created previously.
//
//                   BytesRead - The number of bytes to read from the
//                   tagged data store node.
//
//                   ReadOffset - The offset into the node where to
//                   start reading from.
//
//  Arguments Out:   pBuffer - The buffer to copy the data into.
//
//                   BytesRead - The number of bytes actually read from
//                   the tagged store node.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_PARAM_INVALID
//                   NWE_TDS_INVALID_TAG
//
//  Abstract:        This API reads data from a tagged data store node.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcReadTds
{
         unsigned long      uTag;
         unsigned long      BytesToRead;
         unsigned long      ReadOffset;
         void				*pBuffer;
         unsigned long      BytesRead;

} NwcReadTds, *PNwcReadTds;

//++=======================================================================
//  API Name:        NwcRawRequest
//
//  Arguments In:    ConnHandle - The connection handle of the connection
//                   that the request is being directed to.
//
//                   uFunction - The NCP function that is being called.
//
//                   uNumRequestFrags - The number of fragments that the
//                   request packet has been broken into.
//
//                   pRequestFrags - List of fragments that make up the
//                   request packet.  Each fragment includes the length
//                   of the fragment data and a pointer to the data.
//
//                   uNumReplyFrags - The number of fragments the reply
//                   packet has been broken into.
//
//  Arguments Out:   pReplyFrags - List of fragments that make up the
//                   request packet.  Each fragment includes the length
//                   of the fragment data and a pointer to the data.
//
//                   uActualReplyLength - Total size of the reply packet
//                   after any header and tail information is removed.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//
//  Abstract:        API for sending raw NCP packets directly to a server.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcRequest
{
         NW_CONN_HANDLE       ConnHandle;
         unsigned long        uFunction;
         unsigned long        uNumRequestFrags;
         PNwcFrag             pRequestFrags;
         unsigned long        uNumReplyFrags;
         PNwcFrag             pReplyFrags;
         unsigned long        uActualReplyLength;

} NwcRequest, *PNwcRequest;

//++=======================================================================
//  API Name:        NwcRawRequestAll
//
//  Arguments In:    uFunction - The NCP function that is being called.
//
//                   uNumRequestFrags - The number of fragments that the
//                   request packet has been broken into.
//
//                   pRequestFrags - List of fragments that make up the
//                   request packet.  Each fragment includes the length
//                   of the fragment data and a pointer to the data.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                  
//
//  Abstract:        API for sending the given NCP request to all valid
//                   connections.  If there is a private connection that
//                   is not owned by the caller of this function, that
//                   connection will not be included.  Also, if the
//                   caller has both a private and a public connection
//                   to the same server, only the private connection
//                   will receive the request.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcRequestAll
{
         unsigned long       uFunction;
         unsigned long       uNumRequestFrags;
         PNwcFrag            pRequestFrags;

} NwcRequestAll, *PNwcRequestAll;

//++=======================================================================
//  API Name:        NwcScanConnInfo
//
//  Arguments In:    uScanIndex - The index to be used on the next
//                   iteration of the scan.  This value should be initially
//                   set to zero.  The output of this parameter will be
//                   used in subsequent calls to this function.
//
//                   uScanInfoLevel - Describes the composition of the
//                   pScanConnInfo pointer.  If this parameter contains
//                   NWC_CONN_INFO_RETURN_ALL, information for all
//                   connections will be returned.
//
//                   pScanConnInfo - This parameter is a pointer to
//                   data that describes one piece of connection
//                   information.  The type of this data depends on
//                   which level of information is being scanned for.
//                   For instance, if the scan is being used to find all
//                   connections with a particular authentication state,
//                   pScanConnInfo would be a "pnuint" since
//                   authentication state is described as nuint in the
//                   NwcConnInfo structure.
//
//                   uScanFlag - This parameter tells whether to return
//                   connection information for connections that match
//                   the scan criteria or that do not match the scan
//                   criteria.  If the caller wants to find all the
//                   connections that are not in the "NOVELL_INC" DS
//                   tree, he would use the call as described below in
//                   the description except the uScanFlag parameter would
//                   have  the value of NWC_MATCH_NOT_EQUALS.  This flag
//                   is also used to tell the requester whether to
//                   return private or public, licensed or unlicensed
//                   connections.
//
//                   uReturnInfoLevel - Specifies what information
//                   should be returned.
//
//                   uReturnInfoLength - The size in bytes of pConnInfo.
//
//  Arguments Out:   uConnectionReference - Connection reference
//                   associated with the information that is being
//                   returned.
//
//                   pReturnConnInfo - A pointer to the NwcConnInfo
//                   structure defined above.  In some of the
//                   structures within the union, there are pointers to
//                   data to be returned.  It is the responsibility of
//                   the caller to provide pointers to valid memory
//                   to copy this data into.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_RESOURCE_LOCK
//                   NWE_CONN_INVALID
//                   NWE_INVALID_LEVEL
//                   NWE_STRING_TRANSLATION
//                   NWE_INVALID_MATCH_DATA
//                   NWE_MATCH_FAILED
//                   NWE_BUFFER_OVERFLOW
//                   NWE_NO_MORE_ENTRIES
//
//  Abstract:        This API is used to return connection information
//                   for multiple connections.  It will return one
//                   piece or the full structure of connection information
//                   for one connection at a time.  This call is designed
//                   to scan for connections based on any piece of
//                   connection information as described in the
//                   NwcConnInfo structure.  For instance, if the caller
//                   wants to scan for all connections in the DS tree
//                   "NOVELL_INC", the call would be made with the
//                   following paramters:
//
//                      uScanLevelInfo = NWC_CONN_INFO_TREE_NAME
//                      pScanConnInfo = "NOVELL_INC"
//                      uScanFlag = NWC_MATCH_EQUALS |
//                                  NWC_RETURN_PUBLIC |
//                                  NWC_RETURN_LICENSED
//
//                   The scan flag is used to tell if the scan is
//                   supposed to return connections that match or don't
//                   match.  This design doesn't allow any other
//                   conditions for this flag (such as greater than or
//                   less than).
//
//                   If the caller specifies the uReturnInfoLevel =
//                   NWC_CONN_INFO_RETURN_ALL, the full NwcConnInfo
//                   structure is returned.  The caller must supply
//                   data for any pointers in the NwcConnInfo structure
//                   (these include tree name, workgroup id, server name
//                   and transport address).  However if the caller
//                   doesn't want to get a particular piece of info
//                   that is expecting a pointer to some data, a NULL
//                   pointer may be used to indicate ot the requester
//                   that it should not return that piece of information.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcScanConnInfo
{
     unsigned long        uScanIndex;
     unsigned long        uScanInfoLevel;
     unsigned long        uScanInfoLen;
     void                 *pScanConnInfo;
     unsigned long        uScanFlags;
     unsigned long        uReturnInfoLevel;
     unsigned long        uReturnInfoLength;
     unsigned long        uConnectionReference;
     void                 *pReturnConnInfo;

} NwcScanConnInfo, *PNwcScanConnInfo;

//++=======================================================================
//  API Name:        NwcSetConnInfo
//
//  Arguments In:    ConnHandle - Connection handle for the connection to
//                   set information on.
//                   uInfoLevel - Specifies what information should be set.
//
//                   uInfoLen - Length in bytes of the information being set.
//
//                   pConnInfo - Connection information to set.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_RESOURCE_LOCK
//                   NWE_CONN_INVALID
//                   NWE_INVALID_LEVEL
//
//
//  Abstract:        This API sets information in the connection associated
//                   with the connection handle.
//
//  Notes:           At this time the only setable information levels are:
//                      NWC_CONN_INFO_AUTH_STATE
//                      NWC_CONN_INFO_BCAST_STATE
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetConnInfo
{
        NW_CONN_HANDLE      ConnHandle;
        unsigned long       uInfoLevel;
        unsigned long       uInfoLength;
        void				*pConnInfo;

} NwcSetConnInfo, *PNwcSetConnInfo;

//++=======================================================================
//  API Name:        NwcSetDefaultNameContext
//
//  Arguments In:    uNameLength - The length in bytes of the name
//                   context string.
//
//                   pNameContext - The string to be used as the default
//                   name context.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_PARAM_INVALID
//                   NWE_RESOURCE_LOCK
//                   NWE_STRING_TRANSLATION
//
//  Abstract:        This API sets the default name context.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetDefaultNameContext
{
		unsigned long		uTreeLength;
		unsigned char		*pTreeName;
		unsigned long       uNameLength;
		unsigned char		*pNameContext;

} NwcSetDefaultNameContext, *PNwcSetDefaultNameContext;

//++=======================================================================
//  API Name:        NwcSetPreferredDsTree
//
//  Arguments In:    uTreeLength - The length in bytes of the DS tree name.
//
//                   pDsTreeName - The string to be used as the preferred
//                   DS tree name.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_INSUFFICIENT_RESOURCES
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API sets the preferred DS tree name.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetPreferredDsTree
{
         unsigned long      uTreeLength;
         unsigned char      *pDsTreeName;

} NwcSetPreferredDsTree, *PNwcSetPreferredDsTree;

//++=======================================================================
//  API Name:        NwcSetPreferredServer
//
//  Arguments In:    uServerNameLength - The length in bytes of the
//                   preferred server string.
//
//                   pServerName - a pointer to an ASCIIZ string of the
//                   preferred bindery server.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_INSUFFICIENT_RESOURCES
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API sets the preferred server name.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetPreferredServer
{
    unsigned long		uServerNameLength;
    char				*pServerName;

} NwcSetPreferredServer, *PNwcSetPreferredServer;

//++=======================================================================
//  API Name:        NwcSetPrimaryConnection
//
//  Arguments In:    ConnHandle - Connection handle associated to the
//                   connection reference which the caller wishes to set
//                   as primary.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_PRIMARY_NOT_SET
//
//  Abstract:        This API sets the primary connection according to
//                   the connection handle passed in by the caller.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetPrimaryConnection
{
         NW_CONN_HANDLE       ConnHandle;

} NwcSetPrimaryConnection, *PNwcSetPrimaryConnection;

//++=======================================================================
//  API Name:        NwcSetPrinterStrings
//
//  Arguments In:    uLptDevice - The redirected LPT device to set the
//                   printer setup strings for.
//
//                   uSetupStringLength - The length of the setup string.
//
//                   pSetupString - The string that is sent to the print
//                   queue on this redirected device at the time the
//                   Lpt device is opened.  This field can be NULL if
//                   the caller does not want to change the string.
//
//                   uResetStringLength - The length of the reset string.
//
//                   pResetString - The string that is sent to the print
//                   queue at the end of a print job.  This field can be
//                   NULL if the caller does not want to change the
//                   string.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_DEVICE_NOT_REDIRECTED
//                   NWE_INSUFFICIENT_RESOURCES
//
//  Abstract:        This API is used to set the printer setup and reset
//                   strings for the given redirected device.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetPrinterStrings
{
         unsigned long               uLptDevice;
         unsigned long               uSetupStringLength;
         unsigned char               *pSetupString;
         unsigned long               uResetStringLength;
         unsigned char               *pResetString;

} NwcSetPrinterStrings, *PNwcSetPrinterStrings;

//++=======================================================================
//  API Name:        NwcStartCapture
//
//  Arguments In:    ConnHandle - The connection that this LPT device will
//                   be redirected to.
//
//                   uLptDevice - The number of the Lpt device to be
//                   redirected.
//
//                   uQueueId - Print jobs being written to the given
//                   device will be placed in the queue associated with
//                   this Id.
//
//                   pQueueName - The name of the print queue to redirect
//                   output to.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_DEVICE_NOT_REDIRECTED
//
//  Abstract:        This API starts redirection from the given device to
//                   the specified queue or queue Id.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcStartCapture
{
         NW_CONN_HANDLE     ConnHandle;
         unsigned long      uLptDevice;
         unsigned long      uQueueId;
         char				*pQueueName;

} NwcStartCapture, *PNwcStartCapture;

//++=======================================================================
//  API Name:        NwcSysCloseConn
//
//  Arguments In:    ConnHandle - The handle to a connection that is
//                   to be destroyed.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//
//  Abstract:        This API is similiar to the NwcCloseConn API, except
//                   that it forces all handles to the connection closed
//                   and destroys the service connection.  This is a system
//                   level request that will cause all processes that are
//                   accessing this connection to lose access to the
//                   resources associated to the connection.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSysCloseConn
{
         NW_CONN_HANDLE       ConnHandle;

} NwcSysCloseConn, *PNwcSysCloseConn;

//++=======================================================================
//  API Name:        NwcUnlicenseConn
//
//  Arguments In:    ConnHandle - Open connection handle that will be
//                   accessing the connection in an unlicensed manner.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//                   NWE_HANDLE_ALREADY_UNLICENSED
//
//  Abstract:        This API is used to change the state of a connection
//                   handle from licensed to unlicensed.  If all handles
//                   to the connection have been changed to the unlicensed
//                   state, the unlicensed NCP is sent to the server.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcUnlicenseConn
{
         NW_CONN_HANDLE       ConnHandle;

} NwcUnlicenseConn, *PNwcUnlicenseConn;

//++=======================================================================
//  API Name:        NwcUnmapDrive
//
//  Arguments In:    uDriveNumber - The number of the drive to be
//                   unmapped (A=1, B=2, C=3 ...)
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_PARAM_INVALID
//
//  Abstract:        This API deletes a network drive mapping.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcUnmapDrive
{
         unsigned long                uDriveNumber;

} NwcUnmapDrive, *PNwcUnmapDrive;

typedef  struct   tagNwcUnmapDriveEx
{
//         unsigned long      connHdl;
         unsigned long      linkLen;
         char               linkData[1];

} NwcUnmapDriveEx, *PNwcUnmapDriveEx;

//++=======================================================================
//  API Name:        NwcWriteTds
//
//  Arguments In:    uTag - Unique Id of the tag created previously.
//
//                   BytesToWrite - The number of bytes to write into
//                   the tagged data store node.
//
//                   WriteOffset - The offset into the tagged node where
//                   the write should start.
//
//                   pBuffer - The buffer to copy the data from.
//
//  Arguments Out:   BytesWritten - The number of bytes actually written
//                   into the tagged store node.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_PARAM_INVALID
//                   NWE_TDS_INVALID_TAG
//                   NWE_TDS_WRITE_TRUNCATED
//
//  Abstract:        This API, using the unique tag, writes data into
//                   the tagged data store.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcWriteTds
{
   unsigned long        uTag;
   LONG                 BytesToWrite;
   LONG                 WriteOffset;
   PVOID                pBuffer;
   LONG                 BytesWritten;

} NwcWriteTds, *PNwcWriteTds;

//++=======================================================================
//  API Name:        NwcVerifyUncPath
//
//  Arguments In:    pUncPath - Path the caller wishes to verify.
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_DRIVE_BAD_PATH
//
//  Abstract:        This API verifies the existence of a UNC path.
//                   Typically used when a caller is attempting to map
//                   a DOS device (F:, G:, etc).
//
//  Notes:           This API only attempts to verify the path on existing
//                   connections.
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcVerifyUncPath
{
         PCHAR                pUncPath;

} NwcVerifyUncPath, *PNwcVerifyUncPath;

//++=======================================================================
//  API Name:        NwcGetSessionKey
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetSessionKey
{
        NW_CONN_HANDLE       ConnHandle;
        unsigned char        *pSessionKeyComponents;
        unsigned char        *pSessionKey;

} NwcGetSessionKey, *PNwcGetSessionKey;

//++=======================================================================
//  API Name:        NwcCreateSessionKey
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcCreateSessionKey
{
        NW_CONN_HANDLE       ConnHandle;
        unsigned char        *pSessionKey;

} NwcCreateSessionKey, *PNwcCreateSessionKey;

//++=======================================================================
//  API Name:        NwcGetServerVersion
//
//  Arguments In:    ConnHandle - Associated connection handle for the
//                   server to get the version for.
//
//  Arguments Out:   uMajorVersion - Connected server's major version
//                   number.
//
//                   uMinorVersion - Connected server's minor version
//                   number.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//
//  Abstract:        This API is used for retrieving a server's major/minor
//                   version number via an opened connection handle.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetServerVersion
{
        NW_CONN_HANDLE       ConnHandle;
        unsigned long                uMajorVersion;
        unsigned long                uMinorVersion;

} NwcGetServerVersion, *PNwcGetServerVersion;


//++=======================================================================
//  API Name:        NwcQueryFeature
//
//  Arguments In:    Feature - The number associated with a particular
//                   feature that the caller wants to know if the requester
//                   is supporting
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_REQUESTER_FAILURE
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcQueryFeature
{
         unsigned long                Feature;

} NwcQueryFeature, *PNwcQueryFeature;

//++=======================================================================
//  API Name:        NwcGetDefaultNs
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetDefaultNs
{
        unsigned char                VolNum;
        unsigned char                DefaultNameSpace;

} NwcGetDefaultNs, *PNwcGetDefaultNs;

//++=======================================================================
//  API Name:        NWCChangePassword
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcChangeKey
{
         PNwcString           pDomainName;
         unsigned long        AuthType;
         PNwcString           pObjectName;
         unsigned long        NameType;
         unsigned short int   ObjectType;
         PNwcString           pVerifyPassword;
         PNwcString           pNewPassword;

} NwcChangeKey, *PNwcChangeKey;

//++=======================================================================
//  API Name:        NWCEnumerateIdentities
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcEnumerateIdentities
{
        unsigned long        Iterator;
        PNwcString           pDomainName;
        unsigned long        AuthType;
        PNwcString           pObjectName;
        unsigned long        NameType;
        unsigned short int   ObjectType;
        unsigned long        IdentityFlags;
        AUTHEN_ID            AuthenticationId;

} NwcEnumerateIdentities, *PNwcEnumerateIdentities;

//++=======================================================================
//  API Name:        NWCGetIdentityInfo
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetIdentityInfo
{
        AUTHEN_ID            AuthenticationId;
        PNwcString           pDomainName;
        unsigned long        AuthType;
        PNwcString           pObjectName;
        unsigned long        NameType;
        unsigned short int   ObjectType;
        unsigned long        IdentityFlags;

} NwcGetIdentityInfo, *PNwcGetIdentityInfo;

//++=======================================================================
//  API Name:        NWCLoginIdentity
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcLoginIdentity
{
         PNwcString           pDomainName;
         unsigned long        AuthType;
         PNwcString           pObjectName;
         unsigned long        NameType;
         unsigned short int   ObjectType;
         unsigned long        IdentityFlags;
         PNwcString           pPassword;
         AUTHEN_ID            AuthenticationId;

} NwcLoginIdentity, *PNwcLoginIdentity;

//++=======================================================================
//  API Name:        NWCWriteAuthenticationMaterials
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--


typedef  struct   _AUTHEN_CACHE_
{
   unsigned short int                        TotalSize;
   unsigned short int                        PublicKeyLength;
   unsigned short int                        CredentialLength;
   unsigned short int                        SignatureLength;

}AUTHEN_CACHE, *PAUTHEN_CACHE;


typedef  struct   tagNwcWriteAuthMaterials
{
         PNwcString           pDomainName;
         unsigned long        AuthType;
         PNwcString           pObjectName;
         unsigned long        NameType;
         unsigned short int   ObjectType;
         unsigned long        IdentityFlags;
         PAUTHEN_CACHE        authenSize;
         PVOID                authenticationMaterials;
         AUTHEN_ID            AuthenticationId;

} NwcWriteAuthMaterials, *PNwcWriteAuthMaterials;


//++=======================================================================
//  API Name:        NWCLogoutIdentity
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcLogoutIdentity
{
         AUTHEN_ID            AuthenticationId;

} NwcLogoutIdentity, *PNwcLogoutIdentity;

//++=======================================================================
//  API Name:        NWCSetPassword
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetKey
{
         NW_CONN_HANDLE       ConnHandle;
         AUTHEN_ID            AuthenticationId;
         PNwcString           pObjectName;
         unsigned short int               ObjectType;
         PNwcString           pNewPassword;

} NwcSetKey, *PNwcSetKey;

//++=======================================================================
//  API Name:        NWCVerifyPassword
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcVerifyKey
{
         PNwcString           pDomainName;
         unsigned long                AuthType;
         PNwcString           pObjectName;
         unsigned long                NameType;
         unsigned short int               ObjectType;
         PNwcString           pVerifyPassword;

} NwcVerifyKey, *PNwcVerifyKey;

//++=======================================================================
//  API Name:        NWCVerifyPasswordWithConn
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcVerifyKeyWithConn
{
         PNwcString           pObjectName;
         PNwcString           pVerifyPassword;
         NW_CONN_HANDLE       ConnHandle;

} NwcVerifyKeyWithConn, *PNwcVerifyKeyWithConn;

//++=======================================================================
//  API Name:        NwcAuthenticateWithId
//
//  Arguments In:    ConnHandle - The connection to be authenticated
//
//                   AuthenticationId - the authentication Id associated
//                   to the information necessary to authenticate this
//                   connection.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:        This API is used to authenticate a connection using
//                   an authentication ID that has already been created.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcAuthenticateWithId
{
         NW_CONN_HANDLE       ConnHandle;
         AUTHEN_ID            AuthenticationId;

} NwcAuthenticateWithId, *PNwcAuthenticateWithId;

//++=======================================================================
//  API Name:        NwcUnauthenticate
//
//  Arguments In:    ConnHandle - The connection to unauthenticate.
//
//  Arguments Out:   NONE
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//                   NWE_INVALID_OWNER
//                   NWE_RESOURCE_LOCK
//
//  Abstract:        This API removes the authentication for the specified
//                   connection.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcUnauthenticate
{
         NW_CONN_HANDLE       ConnHandle;
         AUTHEN_ID            AuthenticationId;

} NwcUnauthenticate, *PNwcUnauthenticate;

//++=======================================================================
//  API Name:        NwcGetSecurityFlags
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetSecurityFlags
{
        unsigned long                EnabSecurityFlags;
        unsigned long                PrefSecurityFlags;
        unsigned long                ReqSecurityFlags;

} NwcGetSecurityFlags, *PNwcGetSecurityFlags;

//++=======================================================================
//  API Name:        NwcSetSecurityFlags
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetSecurityFlags
{
         unsigned long                PrefSecurityFlags;
         unsigned long                ReqSecurityFlags;

} NwcSetSecurityFlags, *PNwcSetSecurityFlags;

//++=======================================================================
//  API Name:        NwcRenegotiateSecurityLevel
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:           VLM/NETX support only
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcRenegotiateSecurityLevel
{
         NW_CONN_HANDLE       ConnHandle;
         unsigned long                SecurityFlags;

} NwcRenegotiateSecurityLevel, *PNwcRenegotiateSecurityLevel;

//++=======================================================================
//  API Name:        NwcSetConnectionAuthenticated
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:           VLM/NETX support only
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetConnAuthenticated
{
         NW_CONN_HANDLE       ConnHandle;
         unsigned long                UserObject;

} NwcSetConnAuthenticated, *PNwcSetConnAuthenticated;

//++=======================================================================
//  API Name:        NwcResolveObjectComponents
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcResolveObjectComponents
{
         PNwcString           pSrcPath;
         unsigned long                resolutionType;
         NW_CONN_HANDLE       connHandle;
        unsigned long                dsPathLength;
        PNwcString           pBaseClass;
        PNwcString           pUncPath;
        unsigned long                connReference;

} NwcResolveObjectComponents, *PNwcResolveObjectComponents;

//++=======================================================================
//  API Name:        NwcGetRequesterLinkStatus
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct
{
        unsigned long                LinkStatus;

} NwcGetRequesterLinkStatus, *PNwcGetRequesterLinkStatus;

//++=======================================================================
//  API Name:        NwcGetCfgNameServiceProviders
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct
{
        unsigned long                providerCount;
        unsigned long                providers[MAX_NAME_SERVICE_PROVIDERS];

} NwcGetCfgNameServiceProviders, *PNwcGetCfgNameServiceProviders;

//++=======================================================================
//  API Name:        NwcGetUserScope
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcGetUserScope
{
        unsigned long        serviceFlag;
        PNwcString           pObjectName;
        PNwcString           pObjectContext;
        PNwcString           pPassword;

} NwcGetUserScope, *PNwcGetUserScope;

//++=======================================================================
//  API Name:        NwcSetScopeToUser
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcSetScopeToUser
{
         PVOID                pUserLogonId;
         unsigned long        serviceFlag;
         PNwcString           pObjectName;
         PNwcString           pObjectContext;
         PNwcString           pPassword;

} NwcSetScopeToUser, *PNwcSetScopeToUser;

//++=======================================================================
//  API Name:        NwcRemoveUserScope
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcRemoveUserScope
{
         PVOID                pUserLogonId;

} NwcRemoveUserScope, *PNwcRemoveUserScope;

//++=======================================================================
//  API Name:        NwcMapDrivePreNotify
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcMapDrivePreNotify
{
         NW_CONN_HANDLE       ConnHandle;

} NwcMapDrivePreNotify, *PNwcMapDrivePreNotify;

//++=======================================================================
//  API Name:        NwcUnMapDrivePostNotify
//
//  Arguments In:
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//
//  Abstract:
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcUnMapDrivePostNotify
{
         NW_CONN_HANDLE       ConnHandle;

} NwcUnMapDrivePostNotify, *PNwcUnMapDrivePostNotify;

//++=======================================================================
//  API Name:        NwcTest
//
//  Arguments In:    NwcTestPacket
//
//  Arguments Out:
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_PARAM_INVALID
//
//  Abstract:        This API is used for testing the connection model
//                   of the NT redirector.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagConnHandleData
{
   unsigned long        ConnHandle;
   unsigned long        ProcessId;
   unsigned long        Task;
   unsigned long        ReferenceCount;
   unsigned long        Accesses;

} ConnHandleData, *PConnHandleData;

typedef struct _ping_buffer
{
	unsigned long	address;
	unsigned long	cost;

} PingAddressBuffer;

//
// Currently defined test functions
//

#ifdef DBG_WINNT_NWFS
/*
#define  TEST_GET_HANDLES_ON_REFERENCE       0x00000001
#define  TEST_MODIFY_DEBUG_LEVEL             0x00000004
#define  TEST_OPEN_DEBUG_TRACE               0x00000005
#define  TEST_READ_DEBUG_TRACE               0x00000006
#define  TEST_CLOSE_DEBUG_TRACE              0x00000007
#define  TEST_DUMP_DEBUG_TRACE				 0x00000008
#define  TEST_GET_TASK_ID					 0x00000009
#define  NWDBG_BADSERVER_LIST				 0x0000000A
#define  NWDBG_FLUSH_SERVER_LIST			 0x0000000B
#define  NWDBG_SERVER_CACHE					 0x0000000C
#define  NWDBG_NDS_CACHE					 0x0000000D
#define	 NWDBG_COST_CACHE					 0x0000000E
#define  NWDBG_FCB_LIST						 0x0000000F
#define  NWDBG_BAD_ADDRESS_CACHE		     0x00000010
#define  NWDBG_RESET_WS_CACHE				 0x00000011
#define	 NWDBG_GET_CONN_INFO				 0x00000012
#define  NWDBG_DUMP_DEVICE_LIST              0x00000013
#define  NWDBG_DEBUGGER_OFF					 0x00000014
#define  NWDBG_DEBUGGER_ON					 0x00000015
#define  NWDBG_MEMLIST					     0x00000016
#define	 NWDBG_VOLINFO						 0x00000017
#define	 NWDBG_AUTH_LIST                     0x00000018
#define	 NWDBG_SYSTEM_INFO					 0x00000019
#define  NWDBG_SCOPE_INFO                    0x0000001A
#define  NWDBG_TRACE_FSDENTRY				 0x0000001B
#define  NWDBG_SET_PACKET_TRACE				 0x0000001C

// Debug Info Structures

typedef struct _srv_cache
{
	char	serverName[256];
	int		addressType;
	unsigned long	serverAddress;

} ServerCache;

typedef struct _nds_cache
{
	char	NDSName[512];
	unsigned long	DT1;
	unsigned long	DT2;
	unsigned long	DT3;

} NDSCache;

typedef struct _adress_cost
{
	unsigned long	address;
	unsigned long	addressType;
	unsigned long	cost;

} IPCostList;

typedef struct _FCB_List
{
	char	server[128];
	char	fileName[512];
	char	path[512];
	unsigned long	openFlags;
	unsigned long	openType;
	unsigned long	openCount;
	unsigned long	openOnNetworkCount;
	unsigned short int	volumeNumber;
	unsigned short int	dirHandle;

} FCBInfo;

typedef struct _tskinfo
{
   HANDLE                        ProcessId;
   unsigned long                         ReferenceCount;

} TASK_INFO;

typedef struct _connection
{
	char	serverName[128];
	unsigned long	connectionFlags;
	unsigned long	AddressType;
	unsigned long	RemoteAddress;
	unsigned long	RedirResourceCount;
	unsigned long	UNCResourceCount;
	unsigned long	ServerType;
	unsigned long	MaxOpenFiles;
	unsigned long	NcpOpenFiles;
	unsigned long	ConnectionScope;
	TASK_INFO	taskList[252];

} NwDbgConnInfo;

typedef struct _DBG_VOLINFO_
{
   unsigned char        VolumeNumber;
   unsigned short int      RemovableMedia;
   unsigned short int      UserSpaceRestricted;
   unsigned long        DirHandle;
   unsigned long        NameSpace;
   unsigned long        StatusFlagBits;
   unsigned long        ReferenceCount;
   unsigned char		VolumeBuffer[128];
   char			serverName[128];


} NwDbgVolInfo;


typedef  struct   _DBG_MAPPED_DEVICE
{

   unsigned long                   DeviceScope;
   unsigned long                   DeviceType;
   unsigned long                   DeviceNumber;
   CHAR                    PathData[512];

} DbgMappedDevice;

typedef struct _MEMORY_TRACK_ 
{

   unsigned long			TotAlloc;
   unsigned long			allocCount;
   unsigned long			freeCount;
   unsigned long			memBlocks;

} DbgMemTrack;

typedef struct _DBG_AUTHINFO_
{
	unsigned long	authIdentity;
	unsigned long	authScope;
	unsigned long	objectType;
	unsigned long	objectId;
	unsigned char	objectName[256];

} NwDbgAuthInfo;

typedef struct _DBG_SCOPEINFO_
{
	unsigned long	hiLUID;
	unsigned long	lowLUID;
	unsigned long	processScope;
	unsigned long	serviceFlag;
	unsigned char	objectName[256];
	unsigned char	objectContext[256];

	
} NwDbgScopeInfo;


typedef  struct   tagNwcTestPacket
{
         unsigned long                uTestFunction;

   //
   // Data specific to each test
   //

   union
   {
      struct
      {
            unsigned long             ConnectionReference;
           unsigned long             HandlesReturned;
           ConnHandleData    Handles[128];

      } GetConnHandles;

      struct
      {
              Handle;
      } OpenDebugTrace;

      struct
      {
               HANDLE            Handle;
               unsigned long             BufferSize;
              Punsigned char            Buffer;
              unsigned long             BytesCopied;
      } ReadDebugTrace;

      struct
      {
               HANDLE            Handle;
      } CloseDebugTrace;

      struct
      {
               unsigned short int           ClearBuffer;
               unsigned short int           SetDebugLevel;
               unsigned long             NewDebugLevel;
              unsigned long             CurrentDebugLevel;
		 IN		  unsigned long				TraceMacro;
		 IN		  unsigned long				debugPacketTrace;

      } ModifyDebugLevel;

      struct
      {
         //
         // Maximum size of the union
         //

         CHAR        Data[4096 - sizeof(unsigned long)];

      } Reserved;

   } Packet;


} NwcTestPacket, *PNwcTestPacket;

*/

#endif

//++=======================================================================
//  API Name:        NwcNdsResolveNameToId
//
//  Arguments In:    connHandle
//                      Specifies connection to use to resolve name with.
//
//                   pName
//                      Points to the name of the NDS entry to resolve.
//
//                   uReqTranType
//                      Specifies the preferred or required transport to
//                      be used.
//
//                   pResolveInfo
//                      Points to the NwcNdsResolveInfo structure
//                      containing information on how the entry is to be
//                      resolved.
//
//  Arguments Out:   pResolveInfo
//                      Points to the NwcNdsResolveInfo structure
//                      containing return information on the resolved
//                      entry.
//
//                   pluEntryId
//                      Points to the resolved name's entry ID.
//
//                   pReferral
//                      Points to the NwcReferral structure which describes
//                      network addresses that can be used to locate other
//                      NDS partitions that contain the entry name.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_CONN_INVALID,
//                   NWE_BUFFER_OVERFLOW,
//                   NWE_TRAN_INVALID_TYPE,
//                   NWE_ACCESS_VIOLATION,
//                   NWE_UNSUPPORTED_TRAN_TYPE,
//                   Nds error code
//
//  Abstract:        This API resolves a NDS entry name.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcNdsResolveNameToId
{
         NW_CONN_HANDLE       connHandle;
         PNwcString           pName;
         unsigned long                uReqTranType;
     PNwcResolveInfo      pResolveInfo;
        unsigned long                entryId;
        PNwcReferral         pReferral;

} NwcNdsResolveNameToId, *PNwcNdsResolveNameToId;


//++=======================================================================
//  API Name:        NwcOrderedRequest
//
//  Arguments In:    uFunction - The NCP function that is being called.
//
//                   uNumRequestFrags - The number of fragments that the
//                   request packet has been broken into.
//
//                   pRequestFrags - List of fragments that make up the
//                   request packet.  Each fragment includes the length
//                   of the fragment data and a pointer to the data.
//
//                   uInverseReqCode - The NCP function that will be called
//                   if the request fails.
//
//                   uNumInverseFrags - The number of fragments the inverse
//                   request packet has been broken into.
//
//                   pReplyFrags - List of fragments that make up the
//                   inverse request packet.  Each fragment includes the length
//                   of the fragment data and a pointer to the data.
//
//  Returns:         STATUS_SUCCESS
//                   NWE_ACCESS_VIOLATION
//                   NWE_CONN_INVALID
//
//  Abstract:        API for sending raw NCP packets directly to a server.
//
//  Notes:
//
//  Environment:     PASSIVE_LEVEL, WIN32
//
//=======================================================================--

typedef  struct   tagNwcOrderedRequest
{
         unsigned long                uReqCode;
         unsigned long                uNumRequestFrags;
         PNwcFrag             pRequestFrags;
         unsigned long                uInverseReqCode;
         unsigned long                uNumInverseFrags;
         PNwcFrag             pInverseFrags;

} NwcOrderedRequest, *PNwcOrderedRequest;

//++=======================================================================
//  API Name:        NwFsdResolveDFSJunctionPath
//
//Pointer to a context record. The
//                   master Irp pointer points to an Irp which
//                   contains a NwcResolveDFSJunction structure.
//
//                   typedef  struct   tagNwcResolveDFSJunction
//                   {
//                            PNwcString           pJunctionPath;
//                           PNwcString           pPhysicalPath;
//                   } NwcResolveDFSJunction, *PNwcResolveDFSJunction;
//
//
//
// Returns:       NCFS_SUCCESS - this means the path is a contains a junction
//                NCFS_PATH_NOT_FOUND path does not contain a junction or is invalid
//
// Abstract:  tests a path to see  if it contains a junction if it does it resolves the path to a physical
//             server/volume path. Otherwise it fails and returns NCFS_PATH_NOT_FOUND
//
// Notes:        
//
// Environment:
//
//=======================================================================--

typedef  struct   tagNwcResolveDFSJunction
{
         PNwcString           pJunctionPath;
        PNwcString           pPhysicalPath;
} NwcResolveDFSJunction, *PNwcResolveDFSJunction;

#endif  // _NWCAPI_

//=========================================================================
//=========================================================================

